/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.renamer;

import com.googlepages.dronten.jtagger.audio.AudioTrack;


/**
 * The file renamer object.
 * Set the public members and call rename.
 */
public abstract class BaseRenamer {
    protected String    aPart1 = null;
    protected String    aPart2 = null;
    protected int       aTrackCount = 0;
    protected boolean   aSplitString = false;


    /**
     * Split string at extension.
     * If true the some string operations will only work with on the string wihout filextension.
     *
     * @param splitString true or false
     */
    protected BaseRenamer(boolean splitString) {
        aSplitString = splitString;
    }


    /**
     * Append text last in filename but before extension.
     *
     * @param name      Filename or text
     * @param append    Append string
     * @return          New string
     */
    protected String appendText(String name, String append) {
        split(name);
        return aPart1 + append + aPart2;
    }


    /**
     * Append tracknumber last in text.
     *
     * @param name          Filename or text
     * @param startStr      Start number
     * @param sizeStr       Width of number
     * @param separator     Separator string
     * @param incStartStr   True to add track count to start
     * @return              New string
     */
    protected String appendNumber(String name, String startStr, String sizeStr, String separator, boolean incStartStr) {
        int start;
        int size;

        split(name);

        try {
            start = Integer.valueOf(startStr);
        }
        catch (NumberFormatException e) {
            start = 1;
        }

        try {
            size = Integer.valueOf(sizeStr);
        }
        catch (NumberFormatException e) {
            size = 2;
        }

        if (incStartStr) {
            start += aTrackCount;
        }

        if (separator == null) {
            separator = "";
        }

        switch (size) {
            case 1:
                return String.format("%s%s%01d%s", aPart1, separator, start, aPart2);

            case 2:
                return String.format("%s%s%02d%s", aPart1, separator, start, aPart2);

            case 3:
                return String.format("%s%s%03d%s", aPart1, separator, start, aPart2);

            case 4:
                return String.format("%s%s%04d%s", aPart1, separator, start, aPart2);

            case 5:
                return String.format("%s%s%05d%s", aPart1, separator, start, aPart2);

            case 6:
                return String.format("%s%s%06d%s", aPart1, separator, start, aPart2);

            default:
                return String.format("%s%s%d%s", aPart1, separator, start, aPart2);
        }
    }


    /**
     * Capitalize string.
     *
     * @param name  Filename or text
     * @param mode  0,1,2,3
     * @return      Capitalized string
     */
    protected String capitalize(String name, int mode) {
        split(name);
        if (mode == 2) {
            return aPart1.toUpperCase() + aPart2;
        }
        else if (mode == 3) {
            return aPart1.toLowerCase() + aPart2;
        }
        else if (mode == 1) {
            String newName = "";
            boolean uppercase = true;
            char c;
            for (int f = 0; f < aPart1.length(); f++) {
                c = name.charAt(f);
                if (Character.isLetter(c)) {
                    if (uppercase) {
                        newName += Character.toUpperCase(c);
                        uppercase = false;
                    }
                    else {
                        newName += Character.toLowerCase(c);
                    }
                }
                else {
                    newName += c;
                }

                if (c == ' ') {
                    uppercase = true;
                }
            }
            return newName + aPart2;
        }
        else {
            return name;
        }
    }


    /**
     * Capitalise extension.
     *
     * @param name  Filename
     * @param mode  0,1,2
     * @return      Capitalized extension
     */
    protected String capitalizeExtension(String name, int mode) {
        if (mode == 1) {
            split(name);
            return aPart1 + aPart2.toUpperCase();
        }
        else if (mode == 2) {
            split(name);
            return aPart1 + aPart2.toLowerCase();
        }
        else {
            return name;
        }
    }


    /**
     * Get next track number.
     *
     * @param startStr  Start value of track number
     * @return          Next sequential track number
     */
    protected String getTrack(String startStr) {
        int start;

        try {
            start = Integer.valueOf(startStr);
        }
        catch (NumberFormatException e) {
            start = 1;
        }
        return String.format("%d", aTrackCount + start);
    }


    /**
     * Insert tracknumber first in text.
     *
     * @param name          Text
     * @param startStr      Start number
     * @param sizeStr       Width of number
     * @param separator     Separator string
     * @param incStartStr   True to add track count to start
     * @return              New string
     */
    protected String insertNumber(String name, String startStr, String sizeStr, String separator, boolean incStartStr) {
        int start;
        int size;

        try {
            start = Integer.valueOf(startStr);
        }
        catch (NumberFormatException e) {
            start = 1;
        }

        try {
            size = Integer.valueOf(sizeStr);
        }
        catch (NumberFormatException e) {
            size = 2;
        }

        if (incStartStr) {
            start += aTrackCount;
        }

        if (separator == null) {
            separator = "";
        }

        switch (size) {
            case 1:
                return String.format("%01d%s%s", start, separator, name);

            case 2:
                return String.format("%02d%s%s", start, separator, name);

            case 3:
                return String.format("%03d%s%s", start, separator, name);

            case 4:
                return String.format("%04d%s%s", start, separator, name);

            case 5:
                return String.format("%05d%s%s", start, separator, name);

            case 6:
                return String.format("%06d%s%s", start, separator, name);

            default:
                return String.format("%d%s%s", start, separator, name);
        }
    }


    /**
     * Remove all numbers and other none letters which appears first in text.
     *
     * @param name  Filename or string
     * @return      New string
     */
    protected String removeLeading(String name) {
        char c;
        String str = "";
        boolean quit = false;

        split(name);

        for (int f = 0; f < aPart1.length(); f++) {
            c = aPart1.charAt(f);

            if (!quit && !Character.isLetter(c)) {
            }
            else {
                quit = true;
                str += c;
            }
        }

        return str + aPart2;
    }


    /**
     * Remove all numbers and other none letters which appears first in text.
     *
     * @param name  Filename or string
     * @return      New string
     */
    protected String removeTrailing(String name) {
        char c;
        String str = "";
        boolean quit = false;

        split(name);
        for (int f = aPart1.length() - 1; f >= 0; f--) {
            c = aPart1.charAt(f);

            if (!quit && !Character.isLetter(c)) {
            }
            else {
                quit = true;
                str = c + str;
            }
        }
        return str + aPart2;
    }


    /**
     * Remove/replace all text.
     *
     * @param text      Original text
     * @param remove    Remove this string
     * @param replace   Replace with this string
     * @return          Text without removed text and optional replaced text
     */
    protected String replace(String text, String remove, String replace) {
        split(text);
        return aPart1.replaceAll(remove, replace) + aPart2;
    }


    /**
     * Set start of track counter
     *
     * @param trackStart Start track numer
     */
    public void setTrackStart(int trackStart) {
        aTrackCount = trackStart;
    }


    /**
     * Override to rename stuff.
     *
     * @param track Audio track
     */
    public abstract void rename(AudioTrack track) throws Exception;


    /**
     * Split filename at ".mp3".
     * Set class variabel aPart1 and aPart2.
     * If string is not an filename then part2 will be empty.
     *
     * @param name Filename
     */
    protected void split(String name) {
        if (aSplitString) {
            String s2 = name.toLowerCase();
            if (s2.endsWith(".mp3") || s2.endsWith(".ogg")) {
                int index = s2.lastIndexOf(".mp3");

                if (index < 0) {
                    index = s2.lastIndexOf(".ogg");
                }

                if (index > 0) {
                    aPart1 = name.substring(0, index);
                    aPart2 = name.substring(index);
                }
            }
            else {
                aPart1 = name;
                aPart2 = "";
            }
        }
        else {
            aPart1 = name;
            aPart2 = "";
        }
    }


    /**
     * Use title as text.
     *
     * @param name  Filename or string
     * @param title Title
     * @return      New String with title
     */
    protected String setString(String name, String title) {
        split(name);
        return title + aPart2;
    }
}
